@@ -1,4 +1,8 @@
-local check = require "luacheck.check"
+local check_full = require "luacheck.check"
+
+local function check(src)
+   return check_full(src).events
+end
 
 describe("check", function()
    it("does not find anything wrong in an empty block", function()
@@ -7,7 +11,7 @@ describe("check", function()
 
    it("does not find anything wrong in used locals", function()
       assert.same({
-         {code = "113", name = "print", line = 5, column = 4, end_column = 8}
+         {code = "113", name = "print", indexing = {"print"}, line = 5, column = 4, end_column = 8}
       }, check[[
 local a
 local b = 5
@@ -20,7 +24,7 @@ end
 
    it("detects global set", function()
       assert.same({
-         {code = "111", name = "foo", line = 1, column = 1, end_column = 3, top = true}
+         {code = "111", name = "foo", indexing = {"foo"}, line = 1, column = 1, end_column = 3, top = true}
       }, check[[
 foo = {}
 ]])
@@ -28,7 +32,7 @@ foo = {}
 
    it("detects global set in nested functions", function()
       assert.same({
-         {code = "111", name = "foo", line = 2, column = 4, end_column = 6}
+         {code = "111", name = "foo", indexing = {"foo"}, line = 2, column = 4, end_column = 6}
       }, check[[
 local function bar()
    foo = {}
@@ -39,9 +43,9 @@ bar()
 
    it("detects global access in multi-assignments", function()
       assert.same({
-         {code = "111", name = "y", line = 2, column = 4, end_column = 4, top = true},
+         {code = "111", name = "y", indexing = {"y"}, line = 2, column = 4, end_column = 4, top = true},
          {code = "532", line = 2, column = 6, end_column = 6},
-         {code = "113", name = "print", line = 3, column = 1, end_column = 5}
+         {code = "113", name = "print", indexing = {"print"}, line = 3, column = 1, end_column = 5}
       }, check[[
 local x
 x, y = 1
@@ -51,8 +55,8 @@ print(x)
 
    it("detects global access in self swap", function()
       assert.same({
-         {code = "113", name = "a", line = 1, column = 11, end_column = 11},
-         {code = "113", name = "print", line = 2, column = 1, end_column = 5}
+         {code = "113", name = "a", indexing = {"a"}, line = 1, column = 11, end_column = 11},
+         {code = "113", name = "print", indexing = {"print"}, line = 2, column = 1, end_column = 5}
       }, check[[
 local a = a
 print(a)
@@ -61,16 +65,105 @@ print(a)
 
    it("detects global mutation", function()
       assert.same({
-         {code = "112", name = "a", line = 1, column = 1, end_column = 1}
+         {code = "112", name = "a", indexing = {"a", false}, line = 1, column = 1, end_column = 1}
       }, check[[
 a[1] = 6
 ]])
    end)
 
+   it("detects indirect global field access", function()
+      assert.same({
+         {
+            code = "113",
+            name = "b",
+            indexing = {"b", false},
+            line = 2,
+            column = 15,
+            end_column = 15
+         }, {
+            code = "113",
+            name = "b",
+            indexing = {"b", false, false, "foo"},
+            previous_indexing_len = 2,
+            line = 3,
+            column = 8,
+            end_column = 12,
+            indirect = true
+         }
+      }, check[[
+local c = "foo"
+local alias = b[1]
+return alias[2][c]
+]])
+   end)
+
+   it("detects indirect global field mutation", function()
+      assert.same({
+         {
+            code = "113",
+            name = "b",
+            indexing = {"b", false},
+            line = 2,
+            column = 15,
+            end_column = 15
+         }, {
+            code = "112",
+            name = "b",
+            indexing = {"b", false, false, "foo"},
+            previous_indexing_len = 2,
+            line = 3,
+            column = 1,
+            end_column = 5,
+            indirect = true
+         }
+      }, check[[
+local c = "foo"
+local alias = b[1]
+alias[2][c] = c
+]])
+   end)
+
+   it("provides indexing information for warnings related to globals", function()
+      assert.same({
+         {
+            code = "113",
+            name = "global",
+            indexing = {"global"},
+            line = 2,
+            column = 11,
+            end_column = 16
+         }, {
+            code = "113",
+            name = "global",
+            indexing = {"global", "foo", "bar", false},
+            indirect = true,
+            previous_indexing_len = 1,
+            line = 3,
+            column = 15,
+            end_column = 15
+         }, {
+            code = "113",
+            name = "global",
+            indexing = {"global", "foo", "bar", false, true},
+            indirect = true,
+            previous_indexing_len = 4,
+            line = 5,
+            column = 8,
+            end_column = 13
+         }
+      }, check[[
+local c = "foo"
+local g = global
+local alias = g[c].bar[1]
+local alias2 = alias
+return alias2[...]
+]])
+   end)
+
    it("detects unused locals", function()
       assert.same({
          {code = "211", name = "a", line = 1, column = 7, end_column = 7},
-         {code = "113", name = "print", line = 5, column = 4, end_column = 8}
+         {code = "113", name = "print", indexing = {"print"}, line = 5, column = 4, end_column = 8}
       }, check[[
 local a = 4
 
@@ -85,7 +178,7 @@ end
       assert.same({
          {code = "211", name = "_", useless = true, line = 2, column = 10, end_column = 10},
          {code = "211", name = "_", useless = true, line = 7, column = 13, end_column = 13},
-         {code = "211", name = "_", filtered = true, secondary = true, line = 12, column = 13, end_column = 13}
+         {code = "211", name = "_", secondary = true, line = 12, column = 13, end_column = 13}
       }, check[[
 do
    local _
@@ -194,7 +287,7 @@ return a
       assert.same({
          {code = "213", name = "i", line = 1, column = 5, end_column = 5},
          {code = "213", name = "i", line = 2, column = 5, end_column = 5},
-         {code = "113", name = "pairs", line = 2, column = 10, end_column = 14}
+         {code = "113", name = "pairs", indexing = {"pairs"}, line = 2, column = 10, end_column = 14}
       }, check[[
 for i=1, 2 do end
 for i in pairs{} do end
@@ -203,25 +296,42 @@ for i in pairs{} do end
 
    it("detects unused values", function()
       assert.same({
-         {code = "311", name = "a", line = 3, column = 4, end_column = 4},
-         {code = "311", name = "a", line = 5, column = 4, end_column = 4},
-         {code = "113", name = "print", line = 9, column = 1, end_column = 5}
+         {code = "311", name = "a", line = 3, column = 4, end_column = 4, overwritten_line = 3, overwritten_column = 7},
+         {code = "311", name = "a", line = 3, column = 7, end_column = 7, overwritten_line = 8, overwritten_column = 1},
+         {code = "311", name = "a", line = 5, column = 4, end_column = 4, overwritten_line = 8, overwritten_column = 1}
       }, check[[
 local a
 if ... then
-   a = 2
+   a, a = 2, 4
 else
    a = 3
 end
 
 a = 5
-print(a)
+return a
+]])
+   end)
+
+   it("does not provide overwriting location if value can reach end of scope", function()
+      assert.same({
+         {code = "311", name = "a", line = 4, column = 4, end_column = 4},
+         {code = "311", name = "a", line = 7, column = 7, end_column = 7}
+      }, check[[
+do
+   local a = 1
+   (...)(a)
+   a = 2
+
+   if ... then
+      a = 3
+   end
+end
 ]])
    end)
 
    it("does not detect unused value when it and a closure using it can live together", function()
       assert.same({
-         {code = "113", name = "escape", line = 3, column = 4, end_column = 9}
+         {code = "113", name = "escape", indexing = {"escape"}, line = 3, column = 4, end_column = 9}
       }, check[[
 local a = 3
 if true then
@@ -250,7 +360,8 @@ return a, b
 
    it("considers a variable initialized if short rhs ends with potential multivalue", function()
       assert.same({
-         {code = "311", name = "b", line = 2, column = 13, end_column = 13, secondary = true}
+         {code = "311", name = "b", line = 2, column = 13, end_column = 13, secondary = true,
+            overwritten_line = 3, overwritten_column = 4}
       }, check[[
 return function(...)
    local a, b = ...
@@ -357,10 +468,14 @@ end
 
    it("detects duplicated fields in table literals", function()
       assert.same({
-         {code = "314", field = "key", line = 3, column = 4, end_column = 4},
-         {code = "314", field = "2", index = true, line = 6, column = 4, end_column = 4},
-         {code = "314", field = "key", line = 7, column = 4, end_column = 6},
-         {code = "314", field = "0.2e1", line = 9, column = 4, end_column = 4}
+         {code = "314", field = "key", line = 3, column = 4, end_column = 4,
+            overwritten_line = 7, overwritten_column = 4},
+         {code = "314", field = "2", index = true, line = 6, column = 4, end_column = 4,
+            overwritten_line = 9, overwritten_column = 4},
+         {code = "314", field = "key", line = 7, column = 4, end_column = 6,
+            overwritten_line = 8, overwritten_column = 4},
+         {code = "314", field = "0.2e1", line = 9, column = 4, end_column = 4,
+            overwritten_line = 10, overwritten_column = 4}
       }, check[[
 local x, y, z = 1, 2, 3
 return {
@@ -378,8 +493,10 @@ return {
 
    it("considers a variable assigned even if it can't get a value due to short rhs (it still gets nil)", function()
       assert.same({
-         {code = "311", name = "a", line = 1, column = 7, end_column = 7},
-         {code = "311", name = "b", line = 1, column = 10, end_column = 10},
+         {code = "311", name = "a", line = 1, column = 7, end_column = 7,
+            overwritten_line = 2, overwritten_column = 1},
+         {code = "311", name = "b", line = 1, column = 10, end_column = 10,
+            overwritten_line = 2, overwritten_column = 4},
          {code = "532", line = 2, column = 6, end_column = 6}
       }, check[[
 local a, b = "foo", "bar"
@@ -390,8 +507,10 @@ return a, b
 
    it("reports vartype == var when the unused value is not the initial", function()
       assert.same({
-         {code = "312", name = "b", line = 1, column = 23, end_column = 23},
-         {code = "311", name = "a", line = 2, column = 4, end_column = 4}
+         {code = "312", name = "b", line = 1, column = 23, end_column = 23,
+            overwritten_line = 4, overwritten_column = 4},
+         {code = "311", name = "a", line = 2, column = 4, end_column = 4,
+            overwritten_line = 3, overwritten_column = 4}
       }, check[[
 local function foo(a, b)
    a = a or "default"
@@ -406,8 +525,8 @@ return foo
 
    it("does not detect unused values in loops", function()
       assert.same({
-         {code = "113", name = "print", line = 3, column = 4, end_column = 8},
-         {code = "113", name = "math", line = 4, column = 8, end_column = 11}
+         {code = "113", name = "print", indexing = {"print"}, line = 3, column = 4, end_column = 8},
+         {code = "113", name = "math", indexing = {"math", "floor"}, line = 4, column = 8, end_column = 11}
       }, check[[
 local a = 10
 while a > 0 do
@@ -433,7 +552,7 @@ goto loop
       assert.same({
          {code = "211", name = "foo", line = 1, column = 7, end_column = 9},
          {code = "411", name = "foo", line = 2, column = 7, end_column = 9, prev_line = 1, prev_column = 7},
-         {code = "113", name = "print", line = 3, column = 1, end_column = 5}
+         {code = "113", name = "print", indexing = {"print"}, line = 3, column = 1, end_column = 5}
       }, check[[
 local foo
 local foo = "bar"
@@ -458,7 +577,8 @@ end
       assert.same({
          {code = "212", name = "self", line = 2, column = 11, end_column = 11, self = true},
          {code = "212", name = "self", line = 4, column = 14, end_column = 14, self = true},
-         {code = "432", name = "self", line = 4, column = 14, end_column = 14, self = true, prev_line = 2, prev_column = 11}
+         {code = "432", name = "self", line = 4, column = 14, end_column = 14, self = true,
+            prev_line = 2, prev_column = 11}
       }, check[[
 local t = {}
 function t:f()
@@ -643,7 +763,7 @@ end
 
    it("detects accessing uninitialized variables", function()
       assert.same({
-         {code = "113", name = "get", line = 6, column = 8, end_column = 10},
+         {code = "113", name = "get", indexing = {"get"}, line = 6, column = 8, end_column = 10},
          {code = "321", name = "a", line = 6, column = 12, end_column = 12}
       }, check[[
 local a
@@ -661,7 +781,7 @@ return a
    it("detects mutating uninitialized variables", function()
       assert.same({
          {code = "341", name = "a", line = 4, column = 4, end_column = 4},
-         {code = "113", name = "get", line = 6, column = 8, end_column = 10}
+         {code = "113", name = "get", indexing = {"get"}, line = 6, column = 8, end_column = 10}
       }, check[[
 local a
 
@@ -677,7 +797,7 @@ return a
 
    it("detects accessing uninitialized variables in nested functions", function()
       assert.same({
-         {code = "113", name = "get", line = 7, column = 8, end_column = 10},
+         {code = "113", name = "get", indexing = {"get"}, line = 7, column = 8, end_column = 10},
          {code = "321", name = "a", line = 7, column = 12, end_column = 12}
       }, check[[
 return function() return function(...)
@@ -696,7 +816,7 @@ end end
 
    it("does not detect accessing unitialized variables incorrectly in loops", function()
       assert.same({
-         {code = "113", name = "get", line = 4, column = 8, end_column = 10}
+         {code = "113", name = "get", indexing = {"get"}, line = 4, column = 8, end_column = 10}
       }, check[[
 local a
 
@@ -769,16 +889,37 @@ return foo;
 ]])
    end)
 
-   it("marks ignored warnings as filtered", function()
+   it("emits events, per-line options, and line lengths", function()
+      assert:set_parameter("TableFormatLevel", math.huge)
       assert.same({
-         {code = "211", name = "foo", filtered = true, line = 1, column = 7, end_column = 9},
-         {code = "211", name = "bar", line = 1, column = 12, end_column = 14},
-         {code = "512", filtered = true, line = 2, column = 1, end_column = 3},
-         {code = "213", name = "_", filtered = true, line = 2, column = 5, end_column = 5},
-         {code = "113", name = "pairs", filtered_113 = true, line = 2, column = 10, end_column = 14},
-      }, check[[
+         events = {
+            {push = true, line = 1, column = 1, end_column = 28},
+            {options = {ignore = {"bar"}}, line = 1, column = 1, end_column = 28},
+            {code = "211", name = "foo", line = 2, column = 7, end_column = 9},
+            {code = "211", name = "bar", line = 2, column = 12, end_column = 14},
+            {pop = true, line = 3, column = 1, end_column = 16},
+            {push = true, closure = true, line = 4, column = 8},
+            {options = {ignore = {".*"}}, line = 5, column = 1, end_column = 19},
+            {code = "512", line = 7, column = 1, end_column = 3},
+            {code = "213", name = "_", line = 7, column = 5, end_column = 5},
+            {code = "113", name = "pairs", indexing = {"pairs"}, line = 7, column = 10, end_column = 14},
+            {pop = true, closure = true, line = 9, column = 1}
+         },
+         per_line_options = {
+            [2] = {{options = {ignore = {"foo"}}, line = 2, column = 16, end_column = 38}}
+         },
+         line_lengths = {28, 38, 16, 17, 19, 17, 32, 16, 3},
+         line_endings = {"comment", "comment", "comment", nil, "comment", "comment", nil, "comment", nil}
+      }, check_full[[
+-- luacheck: push ignore bar
 local foo, bar -- luacheck: ignore foo
-for _ in pairs({}) do return end -- luacheck: ignore
+-- luacheck: pop
+return function()
+-- luacheck: ignore
+-- luacheck: push
+for _ in pairs({}) do return end
+-- luacheck: pop
+end
 ]])
    end)
 
