@@ -47,13 +47,15 @@ local function get_exitcode(command)
    end
 end
 
+-- luacheck: max line length 180
+
 describe("cli", function()
    it("exists", function()
       assert.equal(0, get_exitcode "--help")
    end)
 
    it("handles invalid options", function()
-      assert.equal(3, get_exitcode "--invalid-option")
+      assert.equal(4, get_exitcode "--invalid-option")
    end)
 
    it("works for correct files", function()
@@ -122,18 +124,20 @@ Total: 5 warnings / 0 errors in 1 file
 
    it("detects whitespace issues", function()
       assert.equal([[
-Checking spec/samples/bad_whitespace.lua          8 warnings
+Checking spec/samples/bad_whitespace.lua          10 warnings
 
     spec/samples/bad_whitespace.lua:4:26: line contains trailing whitespace
-    spec/samples/bad_whitespace.lua:8:25: line contains trailing whitespace
-    spec/samples/bad_whitespace.lua:13:40: line contains trailing whitespace
-    spec/samples/bad_whitespace.lua:17:1: line contains only whitespace
-    spec/samples/bad_whitespace.lua:18:1: line contains only whitespace
-    spec/samples/bad_whitespace.lua:19:1: line contains only whitespace
-    spec/samples/bad_whitespace.lua:20:1: line contains only whitespace
-    spec/samples/bad_whitespace.lua:25:1: inconsistent indentation (SPACE followed by TAB)
+    spec/samples/bad_whitespace.lua:8:25: trailing whitespace in a comment
+    spec/samples/bad_whitespace.lua:14:20: trailing whitespace in a string
+    spec/samples/bad_whitespace.lua:17:30: trailing whitespace in a comment
+    spec/samples/bad_whitespace.lua:22:40: trailing whitespace in a comment
+    spec/samples/bad_whitespace.lua:26:1: line contains only whitespace
+    spec/samples/bad_whitespace.lua:27:1: line contains only whitespace
+    spec/samples/bad_whitespace.lua:28:1: line contains only whitespace
+    spec/samples/bad_whitespace.lua:29:1: line contains only whitespace
+    spec/samples/bad_whitespace.lua:34:1: inconsistent indentation (SPACE followed by TAB)
 
-Total: 8 warnings / 0 errors in 1 file
+Total: 10 warnings / 0 errors in 1 file
 ]], get_output "spec/samples/bad_whitespace.lua --no-config")
       assert.equal(1, get_exitcode "spec/samples/bad_whitespace.lua --no-config")
    end)
@@ -191,8 +195,8 @@ Checking spec/samples/unused_code.lua             9 warnings
     spec/samples/unused_code.lua:7:11: unused loop variable 'a'
     spec/samples/unused_code.lua:7:14: unused loop variable 'b'
     spec/samples/unused_code.lua:7:17: unused loop variable 'c'
-    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is unused
-    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is unused
+    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is overwritten on line 14 before use
+    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is overwritten on line 15 before use
     spec/samples/unused_code.lua:21:7: variable 'z' is never accessed
 
 Total: 14 warnings / 0 errors in 3 files
@@ -318,8 +322,8 @@ Checking spec/samples/unused_code.lua             9 warnings
     spec/samples/unused_code.lua:7:11: unused loop variable 'a'
     spec/samples/unused_code.lua:7:14: unused loop variable 'b'
     spec/samples/unused_code.lua:7:17: unused loop variable 'c'
-    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is unused
-    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is unused
+    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is overwritten on line 14 before use
+    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is overwritten on line 15 before use
     spec/samples/unused_code.lua:21:7: variable 'z' is never accessed
 
 Total: 9 warnings / 0 errors in 1 file
@@ -331,8 +335,8 @@ Total: 9 warnings / 0 errors in 1 file
 Checking spec/samples/unused_code.lua             4 warnings
 
     spec/samples/unused_code.lua:5:13: unused variable 'q'
-    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is unused
-    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is unused
+    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is overwritten on line 14 before use
+    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is overwritten on line 15 before use
     spec/samples/unused_code.lua:21:7: variable 'z' is never accessed
 
 Total: 4 warnings / 0 errors in 1 file
@@ -386,7 +390,7 @@ Checking s/samples/absent_code.lua                I/O error
 
 Total: 0 warnings / 1 error in 1 file, couldn't check 1 file
 ]], get_output "spec/samples/python_code.lua s/samples/absent_code.lua --no-config")
-      assert.equal(2, get_exitcode "spec/samples/python_code.lua spec/samples/absent_code.lua --no-config")
+      assert.equal(3, get_exitcode "spec/samples/python_code.lua spec/samples/absent_code.lua --no-config")
    end)
 
    it("expands rockspecs", function()
@@ -520,8 +524,8 @@ Checking spec/samples/bad_flow.lua                6 warnings
 
     spec/samples/bad_flow.lua:1:28: empty if branch
     spec/samples/bad_flow.lua:6:4: empty do..end block
-    spec/samples/bad_flow.lua:12:15: left-hand side of assignment is too long
-    spec/samples/bad_flow.lua:16:15: left-hand side of assignment is too short
+    spec/samples/bad_flow.lua:12:15: right side of assignment has less values than left side expects
+    spec/samples/bad_flow.lua:16:15: right side of assignment has more values than left side expects
     spec/samples/bad_flow.lua:21:7: unreachable code
     spec/samples/bad_flow.lua:25:1: loop is executed at most once
 
@@ -544,12 +548,68 @@ Total: 6 warnings / 0 errors in 1 file
 ]], get_output "spec/samples/redefined.lua --globals each --no-config")
    end)
 
+   it("detects lines that are too long", function()
+      assert.equal([[
+Checking spec/samples/line_length.lua             8 warnings
+
+    spec/samples/line_length.lua:2:1: line is too long (123 > 120)
+    spec/samples/line_length.lua:3:1: line is too long (164 > 120)
+    spec/samples/line_length.lua:8:1: line is too long (134 > 120)
+    spec/samples/line_length.lua:13:1: line is too long (47 > 40)
+    spec/samples/line_length.lua:18:1: line is too long (132 > 120)
+    spec/samples/line_length.lua:22:1: line is too long (85 > 80)
+    spec/samples/line_length.lua:26:1: line is too long (104 > 100)
+    spec/samples/line_length.lua:29:1: line is too long (125 > 120)
+
+Total: 8 warnings / 0 errors in 1 file
+]], get_output "spec/samples/line_length.lua --no-config")
+
+      assert.equal([[
+Checking spec/samples/line_length.lua             7 warnings
+
+    spec/samples/line_length.lua:3:1: line is too long (164 > 130)
+    spec/samples/line_length.lua:8:1: line is too long (134 > 130)
+    spec/samples/line_length.lua:13:1: line is too long (47 > 40)
+    spec/samples/line_length.lua:18:1: line is too long (132 > 130)
+    spec/samples/line_length.lua:22:1: line is too long (85 > 80)
+    spec/samples/line_length.lua:26:1: line is too long (104 > 100)
+    spec/samples/line_length.lua:29:1: line is too long (125 > 120)
+
+Total: 7 warnings / 0 errors in 1 file
+]], get_output "spec/samples/line_length.lua --no-config --max-line-length=130")
+
+      assert.equal([[
+Checking spec/samples/line_length.lua             4 warnings
+
+    spec/samples/line_length.lua:13:1: line is too long (47 > 40)
+    spec/samples/line_length.lua:22:1: line is too long (85 > 80)
+    spec/samples/line_length.lua:26:1: line is too long (104 > 100)
+    spec/samples/line_length.lua:29:1: line is too long (125 > 120)
+
+Total: 4 warnings / 0 errors in 1 file
+]], get_output "spec/samples/line_length.lua --no-config --no-max-line-length")
+
+      assert.equal([[
+Checking spec/samples/line_length.lua             7 warnings
+
+    spec/samples/line_length.lua:2:1: line is too long (123 > 120)
+    spec/samples/line_length.lua:3:1: line is too long (164 > 120)
+    spec/samples/line_length.lua:13:1: line is too long (47 > 40)
+    spec/samples/line_length.lua:18:1: line is too long (132 > 120)
+    spec/samples/line_length.lua:22:1: line is too long (85 > 80)
+    spec/samples/line_length.lua:26:1: line is too long (104 > 100)
+    spec/samples/line_length.lua:29:1: line is too long (125 > 120)
+
+Total: 7 warnings / 0 errors in 1 file
+]], get_output "spec/samples/line_length.lua --no-config --no-max-string-line-length")
+   end)
+
    it("detects issues related to read-only globals", function()
       assert.equal([[
 Checking spec/samples/read_globals.lua            5 warnings
 
     spec/samples/read_globals.lua:1:1: setting read-only global variable 'string'
-    spec/samples/read_globals.lua:2:1: mutating read-only global variable 'table'
+    spec/samples/read_globals.lua:2:1: setting undefined field 'append' of global 'table'
     spec/samples/read_globals.lua:5:1: setting read-only global variable 'bar'
     spec/samples/read_globals.lua:6:1: mutating non-standard global variable 'baz'
     spec/samples/read_globals.lua:6:21: accessing undefined variable 'baz'
@@ -558,12 +618,72 @@ Total: 5 warnings / 0 errors in 1 file
 ]], get_output "spec/samples/read_globals.lua --std=lua52 --globals foo --read-globals bar --no-config")
    end)
 
+   it("detects indirect global indexing", function()
+      assert.equal([[
+Checking spec/samples/indirect_globals.lua        3 warnings
+
+    spec/samples/indirect_globals.lua:2:11-16: accessing undefined variable 'global'
+    spec/samples/indirect_globals.lua:5:1-8: indirectly setting undefined field 'concat.foo.bar' of global 'table'
+    spec/samples/indirect_globals.lua:5:32-37: accessing undefined variable 'global'
+
+Total: 3 warnings / 0 errors in 1 file
+]], get_output "spec/samples/indirect_globals.lua --std=min --ranges --no-config")
+   end)
+
+   it("allows defining fields", function()
+      assert.equal([[
+Checking spec/samples/indirect_globals.lua        2 warnings
+
+    spec/samples/indirect_globals.lua:2:11-16: accessing undefined variable 'global'
+    spec/samples/indirect_globals.lua:5:32-37: accessing undefined variable 'global'
+
+Total: 2 warnings / 0 errors in 1 file
+]], get_output "spec/samples/indirect_globals.lua --std=min --globals table.concat.foo --ranges --no-config")
+   end)
+
+   it("detects issues related to global fields", function()
+      assert.equal([[
+Checking spec/samples/global_fields.lua           13 warnings / 1 error
+
+    spec/samples/global_fields.lua:2:16: indirectly accessing undefined field 'upsert' of global 'table'
+    spec/samples/global_fields.lua:8:1: indirectly setting undefined field 'insert.foo' of global 'table'
+    spec/samples/global_fields.lua:23:7: accessing undefined field 'gfind' of global 'string'
+    spec/samples/global_fields.lua:27:7: accessing undefined field 'find' of global 'string'
+    spec/samples/global_fields.lua:32:7: accessing undefined variable 'server'
+    spec/samples/global_fields.lua:33:7: accessing undefined variable 'server'
+    spec/samples/global_fields.lua:34:1: mutating non-standard global variable 'server'
+    spec/samples/global_fields.lua:35:1: mutating non-standard global variable 'server'
+    spec/samples/global_fields.lua:36:1: mutating non-standard global variable 'server'
+    spec/samples/global_fields.lua:37:7: accessing undefined variable 'server'
+    spec/samples/global_fields.lua:38:1: mutating non-standard global variable 'server'
+    spec/samples/global_fields.lua:40:1: invalid inline option
+    spec/samples/global_fields.lua:41:1: mutating non-standard global variable 'server'
+    spec/samples/global_fields.lua:42:1: mutating non-standard global variable 'server'
+
+Total: 13 warnings / 1 error in 1 file
+]], get_output "spec/samples/global_fields.lua --no-config")
+
+      assert.equal([[
+Checking spec/samples/global_fields.lua           7 warnings
+
+    spec/samples/global_fields.lua:2:16: indirectly accessing undefined field 'upsert' of global 'table'
+    spec/samples/global_fields.lua:8:1: indirectly setting undefined field 'insert.foo' of global 'table'
+    spec/samples/global_fields.lua:23:7: accessing undefined field 'gfind' of global 'string'
+    spec/samples/global_fields.lua:27:7: accessing undefined field 'find' of global 'string'
+    spec/samples/global_fields.lua:34:1: setting undefined field 'foo' of global 'server'
+    spec/samples/global_fields.lua:35:1: setting undefined field 'bar.?' of global 'server'
+    spec/samples/global_fields.lua:37:7: accessing undefined field 'baz.abcd' of global 'server'
+
+Total: 7 warnings / 0 errors in 1 file
+]], get_output "spec/samples/global_fields.lua --config=spec/configs/custom_fields_config.luacheckrc")
+   end)
+
    it("allows showing warning codes", function()
       assert.equal([[
 Checking spec/samples/read_globals.lua            5 warnings
 
     spec/samples/read_globals.lua:1:1: (W121) setting read-only global variable 'string'
-    spec/samples/read_globals.lua:2:1: (W122) mutating read-only global variable 'table'
+    spec/samples/read_globals.lua:2:1: (W142) setting undefined field 'append' of global 'table'
     spec/samples/read_globals.lua:5:1: (W121) setting read-only global variable 'bar'
     spec/samples/read_globals.lua:6:1: (W112) mutating non-standard global variable 'baz'
     spec/samples/read_globals.lua:6:21: (W113) accessing undefined variable 'baz'
@@ -613,8 +733,9 @@ Checking spec/samples/inline_options.lua          8 warnings / 2 errors
 Total: 8 warnings / 2 errors in 1 file
 ]], get_output "spec/samples/inline_options.lua --std=none --no-config")
 
+      -- Inline `enable` option overrides CLI `ignore`.
       assert.equal([[
-Checking spec/samples/inline_options.lua          7 warnings / 2 errors
+Checking spec/samples/inline_options.lua          8 warnings / 2 errors
 
     spec/samples/inline_options.lua:6:16: unused function 'f'
     spec/samples/inline_options.lua:12:4: accessing undefined variable 'qu'
@@ -625,8 +746,9 @@ Checking spec/samples/inline_options.lua          7 warnings / 2 errors
     spec/samples/inline_options.lua:26:1: unpaired push directive
     spec/samples/inline_options.lua:28:4: unpaired pop directive
     spec/samples/inline_options.lua:34:1: empty do..end block
+    spec/samples/inline_options.lua:35:10: empty if branch
 
-Total: 7 warnings / 2 errors in 1 file
+Total: 8 warnings / 2 errors in 1 file
 ]], get_output "spec/samples/inline_options.lua --std=none --ignore=542 --no-config")
 
       assert.equal([[
@@ -652,22 +774,25 @@ Total: 5 warnings / 0 errors in 1 file
 ]], get_output "spec/samples/read_globals_inline_options.lua --std=lua52 --no-config")
 
       assert.equal([[
-Checking spec/samples/read_globals_inline_options.lua 1 warning
+Checking spec/samples/read_globals_inline_options.lua 3 warnings
 
-    spec/samples/read_globals_inline_options.lua:3:16: mutating read-only global variable 'baz'
+    spec/samples/read_globals_inline_options.lua:3:1: setting read-only global variable 'foo'
+    spec/samples/read_globals_inline_options.lua:3:16: setting read-only field '?' of global 'baz'
+    spec/samples/read_globals_inline_options.lua:5:1: setting read-only global variable 'foo'
 
-Total: 1 warning / 0 errors in 1 file
+Total: 3 warnings / 0 errors in 1 file
 ]], get_output "spec/samples/read_globals_inline_options.lua --std=lua52 --read-globals baz --globals foo --no-config")
    end)
 
    it("inline options can use extended stds", function()
       assert.equal([[
-Checking spec/samples/custom_std_inline_options.lua 2 warnings
+Checking spec/samples/custom_std_inline_options.lua 3 warnings
 
     spec/samples/custom_std_inline_options.lua:3:1: accessing undefined variable 'tostring'
+    spec/samples/custom_std_inline_options.lua:6:19: accessing undefined variable 'print'
     spec/samples/custom_std_inline_options.lua:6:25: accessing undefined variable 'it'
 
-Total: 2 warnings / 0 errors in 1 file
+Total: 3 warnings / 0 errors in 1 file
 ]], get_output "spec/samples/custom_std_inline_options.lua --config=spec/configs/custom_stds_config.luacheckrc")
    end)
 
@@ -766,8 +891,8 @@ Checking spec/samples/unused_code.lua             9 warnings
     spec/samples/unused_code.lua:7:11: unused loop variable 'a'
     spec/samples/unused_code.lua:7:14: unused loop variable 'b'
     spec/samples/unused_code.lua:7:17: unused loop variable 'c'
-    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is unused
-    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is unused
+    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is overwritten on line 14 before use
+    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is overwritten on line 15 before use
     spec/samples/unused_code.lua:21:7: variable 'z' is never accessed
 
 Total: 16 warnings / 1 error in 4 files
@@ -777,22 +902,25 @@ Total: 16 warnings / 1 error in 4 files
 
          local cache = utils.read_file(tmpname)
          assert.string(cache)
-         local format_version, good_mtime, bad_mtime, python_mtime = cache:match([[
+
+         -- luacheck: push no max string line length
+         local format_version, good_mtime, bad_mtime, python_mtime = cache:match((([[
 
 (%d+)
 spec/samples/good_code.lua
 (%d+)
-return {}
+return {{},{},{19,0,23,17,3,0,30,25,26,3,0,15},{[4]="comment"}}
 spec/samples/bad_code.lua
 (%d+)
-return {{"112","package",1,1,7},{"211","helper",3,16,21,%[10%]=true},{"212","...",3,23,25},{"111","embrace",7,10,16,%[12%]=true},{"412","opt",8,10,12,7,18},{"113","hepler",9,11,16}}
+local A,B,C="package","embrace","hepler";return {{{"112",A,1,1,7,[23]={A,"loaded",true}},{"211","helper",3,16,21,[10]=true},{"212","...",3,23,25},{"111",B,7,10,16,[11]=true,[23]={B}},{"412","opt",8,10,12,7,18},{"113",C,9,11,16,[23]={C}}},{},{24,0,26,9,3,0,21,31,26,3,0},{[4]="comment"}}
 spec/samples/python_code.lua
 (%d+)
-return {{"011",%[3%]=1,%[4%]=6,%[5%]=15,%[23%]="expected '=' near '__future__'"}}
-]])
+return {{{"011",[3]=1,[4]=6,[5]=15,[12]="expected '=' near '__future__'"}},{},{}}
+]]):gsub("[%[%]]", "%%%0")))
+         -- luacheck: pop
 
          format_version = tonumber(format_version)
-         assert.number(format_version)
+         assert.number(format_version, "Cache string is:" .. cache)
          assert.string(good_mtime)
          assert.string(bad_mtime)
          assert.string(python_mtime)
@@ -806,19 +934,21 @@ return {{"011",%[3%]=1,%[4%]=6,%[5%]=15,%[23%]="expected '=' near '__future__'"}
 %s
 spec/samples/python_code.lua
 %s
-return {{"111", "global", 1, 1}, {"321", "uninit", 6, 8}}
+return {{{"111", "global", 1, 1, [23]={"global"}}, {"321", "uninit", 6, 8}},{},{}}
 spec/samples/good_code.lua
 %s
-return {{"011",[3]=5,[4]=7,[23]="this code is actually bad"}}
+return {{{"011",[3]=5,[4]=7,[12]="this code is actually bad"}},{},{}}
 spec/samples/bad_code.lua
 %s
-return {}]]):format(version, python_mtime, good_mtime, tostring(tonumber(bad_mtime) - 1)))
+return {{},{},{}}]]):format(version, python_mtime, good_mtime, tostring(tonumber(bad_mtime) - 1)))
             fh:close()
          end
 
          write_new_cache("\n"..tostring(format_version))
-         assert.equal(mocked_output, get_output("spec/samples/good_code.lua spec/samples/bad_code.lua spec/samples/python_code.lua spec/samples/unused_code.lua --std=lua52 --no-config --cache "..tmpname))
-         assert.equal(mocked_output, get_output("spec/samples/good_code.lua spec/samples/bad_code.lua spec/samples/python_code.lua spec/samples/unused_code.lua --std=lua52 --no-config --cache "..tmpname))
+         assert.equal(mocked_output,
+            get_output("spec/samples/good_code.lua spec/samples/bad_code.lua spec/samples/python_code.lua spec/samples/unused_code.lua --std=lua52 --no-config --cache "..tmpname))
+         assert.equal(mocked_output,
+            get_output("spec/samples/good_code.lua spec/samples/bad_code.lua spec/samples/python_code.lua spec/samples/unused_code.lua --std=lua52 --no-config --cache "..tmpname))
 
          write_new_cache("\n"..tostring(format_version + 1))
          assert.equal(normal_output, get_output("spec/samples/good_code.lua spec/samples/bad_code.lua spec/samples/python_code.lua --std=lua52 --no-config --cache "..tmpname))
@@ -952,7 +1082,7 @@ spec/samples/python_code.lua:1:6: (E011) expected '=' near '__future__'
    end)
 
    it("expands folders", function()
-      assert.matches("^Total: %d+ warnings / %d+ errors in 21 files\n$", get_output "spec/samples -qqq --no-config")
+      assert.matches("^Total: %d+ warnings / %d+ errors in 23 files\n$", get_output "spec/samples -qqq --no-config --exclude-files spec/samples/global_fields.lua")
    end)
 
    it("uses --include-files when expanding folders", function()
@@ -1061,8 +1191,8 @@ Checking spec/samples/unused_code.lua             7 warnings
     spec/samples/unused_code.lua:7:11: unused loop variable 'a'
     spec/samples/unused_code.lua:7:14: unused loop variable 'b'
     spec/samples/unused_code.lua:7:17: unused loop variable 'c'
-    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is unused
-    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is unused
+    spec/samples/unused_code.lua:13:7: value assigned to variable 'x' is overwritten on line 14 before use
+    spec/samples/unused_code.lua:14:1: value assigned to variable 'x' is overwritten on line 15 before use
 
 Total: 8 warnings / 0 errors in 2 files
 ]], get_output "spec/samples/unused_secondaries.lua spec/samples/unused_code.lua --config=spec/configs/multioverride_config.luacheckrc")
@@ -1097,12 +1227,14 @@ Codes: true
 
          it("uses exclude_files option", function()
             assert.equal(([[
-Checking spec/samples/argparse.lua                7 warnings
+Checking spec/samples/argparse.lua                9 warnings
 Checking spec/samples/compat.lua                  4 warnings
 Checking spec/samples/custom_std_inline_options.lua 3 warnings / 1 error
 Checking spec/samples/global_inline_options.lua   3 warnings
 Checking spec/samples/globals.lua                 2 warnings
+Checking spec/samples/indirect_globals.lua        3 warnings
 Checking spec/samples/inline_options.lua          7 warnings / 2 errors
+Checking spec/samples/line_length.lua             8 warnings
 Checking spec/samples/python_code.lua             1 error
 Checking spec/samples/read_globals.lua            5 warnings
 Checking spec/samples/read_globals_inline_options.lua 3 warnings
@@ -1110,19 +1242,21 @@ Checking spec/samples/redefined.lua               7 warnings
 Checking spec/samples/unused_code.lua             9 warnings
 Checking spec/samples/unused_secondaries.lua      4 warnings
 
-Total: 54 warnings / 4 errors in 14 files
+Total: 67 warnings / 4 errors in 16 files
 ]]):gsub("(spec/samples)/", "%1"..package.config:sub(1, 1)),
-            get_output "spec/samples --config=spec/configs/exclude_files_config.luacheckrc -qq")
+            get_output "spec/samples --config=spec/configs/exclude_files_config.luacheckrc -qq --exclude-files spec/samples/global_fields.lua")
          end)
 
          it("loads exclude_files option correctly from upper directory", function()
             assert.equal([[
-Checking argparse.lua                             7 warnings
+Checking argparse.lua                             9 warnings
 Checking compat.lua                               4 warnings
 Checking custom_std_inline_options.lua            3 warnings / 1 error
 Checking global_inline_options.lua                3 warnings
 Checking globals.lua                              2 warnings
+Checking indirect_globals.lua                     3 warnings
 Checking inline_options.lua                       7 warnings / 2 errors
+Checking line_length.lua                          8 warnings
 Checking python_code.lua                          1 error
 Checking read_globals.lua                         5 warnings
 Checking read_globals_inline_options.lua          3 warnings
@@ -1130,25 +1264,27 @@ Checking redefined.lua                            7 warnings
 Checking unused_code.lua                          9 warnings
 Checking unused_secondaries.lua                   4 warnings
 
-Total: 54 warnings / 4 errors in 14 files
-]], get_output(". --config=spec/configs/exclude_files_config.luacheckrc -qq", "spec/samples/"))
+Total: 67 warnings / 4 errors in 16 files
+]], get_output(". --config=spec/configs/exclude_files_config.luacheckrc -qq --exclude-files global_fields.lua", "spec/samples/"))
          end)
 
          it("combines excluded files from config and cli", function()
             assert.equal([[
-Checking argparse.lua                             7 warnings
+Checking argparse.lua                             9 warnings
 Checking compat.lua                               4 warnings
 Checking custom_std_inline_options.lua            3 warnings / 1 error
 Checking global_inline_options.lua                3 warnings
 Checking globals.lua                              2 warnings
+Checking indirect_globals.lua                     3 warnings
 Checking inline_options.lua                       7 warnings / 2 errors
+Checking line_length.lua                          8 warnings
 Checking python_code.lua                          1 error
 Checking redefined.lua                            7 warnings
 Checking unused_code.lua                          9 warnings
 Checking unused_secondaries.lua                   4 warnings
 
-Total: 46 warnings / 4 errors in 12 files
-]], get_output(". --config=spec/configs/exclude_files_config.luacheckrc -qq --exclude-files " .. quote("./read*"), "spec/samples/"))
+Total: 59 warnings / 4 errors in 14 files
+]], get_output(". --config=spec/configs/exclude_files_config.luacheckrc -qq --exclude-files global_fields.lua --exclude-files " .. quote("./read*"), "spec/samples/"))
          end)
 
          it("allows defining custom stds", function()
@@ -1196,6 +1332,75 @@ Checking spec/samples/globals.lua                 1 warning
 Total: 1 warning / 0 errors in 1 file
 ]], get_output "spec/samples/globals.lua --config=spec/configs/import_config.luacheckrc")
          end)
+
+         describe("global path", function()
+            setup(function()
+               os.rename(".luacheckrc", ".luacheckrc.bak")
+            end)
+
+            teardown(function()
+               os.rename(".luacheckrc.bak", ".luacheckrc")
+            end)
+
+            it("uses global path as fallback if --[no-]config is not used", function()
+               assert.equal([[
+Checking spec/samples/compat.lua                  OK
+
+Total: 0 warnings / 0 errors in 1 file
+]], get_output "spec/samples/compat.lua --default-config=spec/configs/global_config.luacheckrc")
+            end)
+
+            it("detects errors in global path config", function()
+               assert.matches([[
+Critical error: Couldn't load configuration from spec/configs/bad_config.luacheckrc: syntax error %(line 2: .* near 'method_missing'%)
+]], get_output "spec/samples/compat.lua --default-config=spec/configs/bad_config.luacheckrc")
+            end)
+
+            it("does not use global path config if it is missing", function()
+               assert.equal([[
+Checking spec/samples/compat.lua                  2 warnings
+
+    spec/samples/compat.lua:1:2: accessing undefined variable 'setfenv'
+    spec/samples/compat.lua:1:22: accessing undefined variable 'setfenv'
+
+Total: 2 warnings / 0 errors in 1 file
+]], get_output "spec/samples/compat.lua --std=lua52 --default-config=spec/configs/config_404.luacheckrc")
+            end)
+
+            it("does not use global path as fallback if --no-default-config is not used", function()
+               assert.equal([[
+Checking spec/samples/compat.lua                  2 warnings
+
+    spec/samples/compat.lua:1:2: accessing undefined variable 'setfenv'
+    spec/samples/compat.lua:1:22: accessing undefined variable 'setfenv'
+
+Total: 2 warnings / 0 errors in 1 file
+]], get_output "spec/samples/compat.lua --std=lua52 --no-default-config")
+            end)
+
+
+            it("does not use global path as fallback if --config is used", function()
+               assert.equal([[
+Files: 1
+Warnings: 2
+Errors: 0
+Quiet: 0
+Color: false
+Codes: true
+]], get_output "spec/samples/compat.lua --default-config=spec/configs/global_config.luacheckrc --config=spec/configs/cli_specific_config.luacheckrc")
+            end)
+
+            it("does not use global path as fallback if --no-config is used", function()
+               assert.equal([[
+Checking spec/samples/compat.lua                  2 warnings
+
+    spec/samples/compat.lua:1:2: accessing undefined variable 'setfenv'
+    spec/samples/compat.lua:1:22: accessing undefined variable 'setfenv'
+
+Total: 2 warnings / 0 errors in 1 file
+]], get_output "spec/samples/compat.lua --std=lua52 --default-config=spec/configs/global_config.luacheckrc --no-config")
+            end)
+         end)
       end)
 
       describe("error handling", function()
@@ -1203,14 +1408,14 @@ Total: 1 warning / 0 errors in 1 file
             assert.matches([[
 Critical error: Couldn't load configuration from spec/configs/bad_config.luacheckrc: syntax error %(line 2: .*%)
 ]], get_output "spec/samples/empty.lua --config=spec/configs/bad_config.luacheckrc")
-            assert.equal(3, get_exitcode "spec/samples/empty.lua --config=spec/configs/bad_config.luacheckrc")
+            assert.equal(4, get_exitcode "spec/samples/empty.lua --config=spec/configs/bad_config.luacheckrc")
          end)
 
          it("raises critical error on non-existent config", function()
             assert.equal([[
 Critical error: Couldn't find configuration file spec/configs/config_404.luacheckrc
 ]], get_output "spec/samples/empty.lua --config=spec/configs/config_404.luacheckrc")
-            assert.equal(3, get_exitcode "spec/samples/empty.lua --config=spec/configs/config_404.luacheckrc")
+            assert.equal(4, get_exitcode "spec/samples/empty.lua --config=spec/configs/config_404.luacheckrc")
          end)
       end)
 
