@@ -67,6 +67,7 @@ describe("luacheck", function()
             {
                code = "111",
                name = "embrace",
+               indexing = {"embrace"},
                top = true
             },
             {
@@ -75,7 +76,8 @@ describe("luacheck", function()
             },
             {
                code = "113",
-               name = "hepler"
+               name = "hepler",
+               indexing = {"hepler"}
             }
          },
          {
@@ -101,6 +103,7 @@ describe("luacheck", function()
             {
                code = "111",
                name = "embrace",
+               indexing = {"embrace"},
                top = true
             },
             {
@@ -109,7 +112,8 @@ describe("luacheck", function()
             },
             {
                code = "113",
-               name = "hepler"
+               name = "hepler",
+               indexing = {"hepler"}
             }
          },
          {
@@ -137,11 +141,13 @@ describe("luacheck", function()
             {
                code = "111",
                name = "embrace",
+               indexing = {"embrace"},
                top = true
             },
             {
                code = "113",
-               name = "hepler"
+               name = "hepler",
+               indexing = {"hepler"}
             }
          },
          {
@@ -199,7 +205,8 @@ describe("check_strings", function()
          {
             {
                code = "113",
-               name = "foo"
+               name = "foo",
+               indexing = {"foo"}
             }
          },
          {
@@ -243,14 +250,18 @@ describe("check_strings", function()
                name = "self",
                line = 3,
                column = 11,
-               end_column = 11
+               end_column = 11,
+               overwritten_line = 4,
+               overwritten_column = 4
             },
             {
                code = "311",
                name = "self",
                line = 4,
                column = 4,
-               end_column = 7
+               end_column = 7,
+               overwritten_line = 5,
+               overwritten_column = 4
             },
             {
                code = "511",
@@ -401,7 +412,8 @@ return f --[=[
          {
             {
                code = "113",
-               name = "foo"
+               name = "foo",
+               indexing = {"foo"}
             }
          },
          {
@@ -425,7 +437,7 @@ describe("get_report", function()
    end)
 
    it("returns a table with single error event on syntax error", function()
-      local report = strip_locations({luacheck.get_report("return return")})[1]
+      local report = strip_locations({luacheck.get_report("return return").events})[1]
       assert.same({code = "011", msg = "expected expression near 'return'"}, report[1])
    end)
 end)
@@ -450,14 +462,16 @@ describe("process_reports", function()
          {
             {
                code = "113",
-               name = "foo"
+               name = "foo",
+               indexing = {"foo"}
             }
          },
          {},
          warnings = 1,
          errors = 0,
          fatals = 0
-      }, strip_locations(luacheck.process_reports({luacheck.get_report("return foo"), luacheck.get_report("return math")})))
+      }, strip_locations(luacheck.process_reports(
+         {luacheck.get_report("return foo"), luacheck.get_report("return math")})))
    end)
 
    it("uses options", function()
@@ -465,19 +479,22 @@ describe("process_reports", function()
          {
             {
                code = "113",
-               name = "foo"
+               name = "foo",
+               indexing = {"foo"}
             }
          },
          {
             {
                code = "113",
-               name = "math"
+               name = "math",
+               indexing = {"math", "floor"}
             }
          },
          warnings = 2,
          errors = 0,
          fatals = 0
-      }, strip_locations(luacheck.process_reports({luacheck.get_report("return foo"), luacheck.get_report("return math")}, {
+      }, strip_locations(luacheck.process_reports(
+         {luacheck.get_report("return foo"), luacheck.get_report("return math.floor")}, {
          std = "none"
       })))
    end)
@@ -508,17 +525,19 @@ describe("get_message", function()
          label = "fail"
       }))
 
-      assert.equal("value assigned to field 'actual' is unused", luacheck.get_message({
+      assert.equal("value assigned to field 'actual' is overwritten on line 2 before use", luacheck.get_message({
          code = "314",
          name = "unrelated",
-         field = "actual"
+         field = "actual",
+         overwritten_line = 2
       }))
 
-      assert.equal("value assigned to index '42' is unused", luacheck.get_message({
+      assert.equal("value assigned to index '42' is overwritten on line 2 before use", luacheck.get_message({
          code = "314",
          name = "11037",
          field = "42",
-         index = true
+         index = true,
+         overwritten_line = 2
       }))
 
       assert.equal("message goes here", luacheck.get_message({
